define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/json",
    "dojo/Deferred",
    "dojo/on",
    "dojo/topic",
    "epi/dependency",
    "epi-cms/contentediting/ContentActionSupport",
    "epi/shell/command/_Command",
    "epi/shell/widget/dialog/Dialog",
    "epi-cms/ApplicationSettings",
    //Resources
    "epi/i18n!epi/cms/nls/episerver.cms.contentediting.contentdetails.command.accessrights",
    "epi/i18n!epi/shell/nls/admin.security",
    "epi/i18n!epi/shell/nls/edit.editsecurity",
    "epi/i18n!epi/cms/nls/episerver.cms.widget.hierachicallist",
    "epi/i18n!epi/cms/nls/episerver.cms.widget.contentreferences",
    "epi/i18n!epi/shell/nls/button",
    "epi/i18n!epi/nls/episerver.shared",
    //Styles
    "xstyle/css!epi-cms/contentediting/AdminWidgets.css"
],

function (
    declare,
    lang,
    json,
    Deferred,
    on,
    topic,
    dependency,
    ContentActionSupport,
    _Command,
    Dialog,
    ApplicationSettings,
    resources,
    securityResources,
    editsecurityResources,
    hierachicalListResources,
    contentReferencesResources,
    buttonResources,
    sharedResources) {

    function AccessRightsViewModel(store) {

        return {
            onMessage: function (message) {
                // callback
            },
            searchEntities: function (searchPhrase) {
                return store.query({
                    includeRoles: true,
                    includeVisitorGroups: true,
                    name: searchPhrase,
                    includeVirtualRoles: true
                }).then(function (results) {
                    return results.map(function (x) {
                        return {
                            key: x.name,
                            entityType: x.reviewerType
                        };
                    });
                });
            }
        };
    }

    return declare([_Command], {
        // summary:
        //      Toggles permanent in use notification on/off.
        //
        // tags:
        //      internal

        name: "AccessRights",
        label: resources.label,
        tooltip: resources.tooltip,
        store: null,
        userNotificationsStore: null,

        constructor: function (params) {
            declare.safeMixin(this, params);
            this.store = this.store || dependency.resolve("epi.storeregistry").get("epi.cms.accessrights");
            this.userNotificationsStore = this.userNotificationsStore || dependency.resolve("epi.storeregistry").get("epi.cms.notification.users");
        },

        _execute: function () {
            require(["epi-cms/contentediting/AdminWidgets"], function (AdminWidgets) {
                var resources = Object.assign({}, securityResources, editsecurityResources, buttonResources, hierachicalListResources, contentReferencesResources, {
                    heading: lang.replace(editsecurityResources.heading, [this.model.contentData.name]),
                    user: lang.replace(editsecurityResources.user, [ApplicationSettings.userName])
                });
                var model = AccessRightsViewModel(this.userNotificationsStore);

                this.store.get(this.model.contentData.contentLink).then(function (settings) {
                    var content = new AdminWidgets.ContentAccessRights({
                        model: model,
                        resources: resources,
                        helpLink: ApplicationSettings.userGuideUrl + "#editsecurity",
                        entries: settings.entries,
                        isInherited: settings.isInherited,
                        isInheritedEnabled: settings.isInheritedEnabled,
                        userGroups: settings.userGroups
                    });
                    var dialog = new Dialog({
                        dialogClass: "epi-dialog-portrait epi-dialog-portrait__autosize epi-dialog--wide",
                        defaultActionsVisible: true,
                        confirmActionText: sharedResources.action.save,
                        content: content,
                        title: editsecurityResources.title
                    });

                    dialog.addValidator(function () {
                        var deferred = new Deferred();
                        this.store.executeMethod("Save", this.model.contentData.contentLink, {
                            entries: model.entries,
                            id: this.model.contentData.contentLink,
                            isInherited: model.isInherited
                        }).then(function () {
                            model.onMessage(editsecurityResources.savemessage);
                            dialog.hide();
                            topic.publish("/epi/cms/contentdata/updated", {
                                contentLink: this.model.contentData.contentLink,
                                recursive: true
                            });
                            return deferred.resolve([]);
                        }.bind(this)).otherwise(function (response) {
                            model.onMessage(response.responseText ? json.fromJson(response.responseText) : "");
                        });
                        return deferred.promise;
                    }.bind(this));

                    dialog.show();

                }.bind(this));
            }.bind(this));
        },

        _onModelChange: function () {
            // summary:
            //		Updates canExecute and isAvailable after the model has been updated.
            // tags:
            //		protected

            var contentData = this.model.contentData,
                hasAdminAccess = ContentActionSupport.hasAccess(contentData.accessMask, ContentActionSupport.accessLevel.Administer);

            this.set("canExecute", contentData.capabilities.securable && hasAdminAccess);
        }
    });
});
